﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace ADM2Sample
{
    public partial class MainForm : Form
    {
        private Boolean[] DevStatus = new Boolean[16];// Device Connection Flag TRUE : connecting
        private DevForm[] IdDevForm = new DevForm[16];// Device Control Panel Form Object
        public MainForm()
        {
            InitializeComponent();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            short DevId;

            Timer1.Interval = 300;
            Timer1.Enabled = true;

            DevIdSel.SelectedIndex = 0;

            // Initialize
            for( DevId = 0 ; DevId < 16 ; DevId++ )
            {
                DevStatus[DevId] = false; // Device is not Connected
            }
        }

        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            short DevId;

            Timer1.Enabled = false;
            for( DevId = 0 ; DevId < 16 ; DevId++ )
            {
                if (IdDevForm[DevId] != null)
                {
                    if (IdDevForm[DevId].Visible)
                    {
                        IdDevForm[DevId].Close();
                        DevStatus[DevId] = false;
                        TUSBADM2.Tusbadm2_Device_Close(DevId);
                    }
                }
            }
        }

        private void FileConvBtn_Click(object sender, EventArgs e)
        {
            string ReadFileName;
            string WriteFileName;
            BinaryReader ReadFile;//読み出しファイルストリーム
            StreamWriter WriteFile;// 保存ファイルストリーム
            byte tmpByte;

            do{
                OpenFileDialog1.FileName= "";
                OpenFileDialog1.Title = "バイナリファイル指定";
                OpenFileDialog1.Filter = "バイナリファイル (*.dat)|*.dat";
                if (OpenFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.Cancel)
                {
                    return;
                }
            }while(OpenFileDialog1.FileName == "" );
            ReadFileName = OpenFileDialog1.FileName;

            do{
                SaveFileDialog1.OverwritePrompt = true;
                SaveFileDialog1.FileName = "";
                SaveFileDialog1.Title = "変換先ファイル指定";
                SaveFileDialog1.Filter = "テキスト保存ファイル (*.csv)|*.csv";

                if(SaveFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.Cancel)
                    return;
            }while(SaveFileDialog1.FileName == "");
            WriteFileName = SaveFileDialog1.FileName;

            ReadFile = new BinaryReader(File.OpenRead(ReadFileName)); //ファイルを開く
            if( File.Exists(SaveFileDialog1.FileName)) // 既存ファイルチェック
                File.Delete(SaveFileDialog1.FileName);
            WriteFile = new StreamWriter(File.OpenWrite(WriteFileName)); // ファイルを開く

            WriteFile.WriteLine("Start date time" + "," + System.DateTime.FromBinary(ReadFile.ReadInt64()).ToString()); // 日時
            WriteFile.WriteLine("Channel" + "," + ReadFile.ReadByte().ToString()); // ch
            WriteFile.WriteLine("Clock select" + "," + ( ReadFile.ReadByte() == 0 ? "Internal": "External")); // クロック選択
            WriteFile.WriteLine("Dividing rate" + "," +  ReadFile.ReadInt32().ToString()); // 内部クロック分周率 n=2～5000
            WriteFile.WriteLine("Ch2 clock invert" + "," +  (ReadFile.ReadByte() == 0 ? "Disable": "Enable")); // Ch2クロック反転　0:なし 1:あり
            WriteFile.WriteLine("Input range" + "," +  (ReadFile.ReadByte() == 0 ? "Unipolar": "Bipolar")); // レンジ 0:ユニポーラ　1:バイポーラ
            tmpByte = ReadFile.ReadByte(); // トリガ選択 0:外部 1:立ち上がり 2:立下り 3:ソフトウェア
            WriteFile.WriteLine("Trigger select" + "," +  (tmpByte == 0 ? "External" : (tmpByte == 1 ? "Up edge" : (tmpByte == 2 ? "Down edge": "Software"))));
            WriteFile.WriteLine("Trigger threshold" + "," +  ReadFile.ReadInt32().ToString()); // アナログトリガ閾値
            WriteFile.WriteLine("Noise cancel level" + "," +  ReadFile.ReadInt32().ToString()); // ノイズキャンセルレベル
            WriteFile.WriteLine("Sampling mode" + "," +  (ReadFile.ReadByte() == 0 ? "Continuation": "Repeat")); // 取り込み動作モード 0:連続 1:リピート
            WriteFile.WriteLine("Sampling length per one trigger" + "," +  ReadFile.ReadInt32().ToString()); // 1トリガ当たりの取り込み長
            WriteFile.WriteLine("Pre trigger length" + "," +  ReadFile.ReadInt32().ToString()); // プレトリガ長

            try{
                do{
                    WriteFile.WriteLine(ReadFile.ReadInt16() & 0xFFF); // オーバーレンジビットはマスクします
                }while(true);
            }
            catch (Exception ex)//ファイルエンド
            {
                if (!(ex is EndOfStreamException))
                {
                    MessageBox.Show(ex.ToString());
                }
            }

            ReadFile.Close();
            WriteFile.Close();

            MessageBox.Show("変換が終了しました");
        }

        // Device Control Panel Open
        private void OpenPanel(short Id)
        {
            DevStatus[Id] = true;
            IdDevForm[Id] = new DevForm();// Device Control Panel Form Create
            IdDevForm[Id].SetId(Id);
            IdDevForm[Id].Show();// Form Show
        }

        private void OpenBtn_Click(object sender, EventArgs e)
        {
            short DevId;

            DevId = (short) DevIdSel.SelectedIndex;
            if( DevStatus[DevId] == false )// Device is not connected
            {
                if( TUSBADM2.Tusbadm2_Device_Open(DevId) == 0 )// Connection check
                    OpenPanel(DevId);
            }
        }

        private void Timer1_Tick(object sender, EventArgs e)
        {
            short DevId;

            for( DevId = 0 ; DevId < 16 ; DevId++ )
            {
                if( DevStatus[DevId] )
                {
                    if( IdDevForm[DevId].Visible == false ) // Device is not visible
                    {
                        TUSBADM2.Tusbadm2_Device_Close(DevId);
                        DevStatus[DevId] = false;
                    }
                }
            }
        }

        private void EndAppBtn_Click(object sender, EventArgs e)
        {
            this.Close();
        }
    }
}
